#!/usr/bin/env bash
####################################################################
#
# This script launches Areca's Text user interface.
#
####################################################################

LINE='----------------------------------------------------'
echo ${LINE}
echo "Launching Areca ..."
echo ${LINE}

#Options
JAVADIR=/usr/java

#Getting Areca's directory
PROGRAM_DIR=`dirname "$0"`
PROGRAM_DIR=`cd "$PROGRAM_DIR"; pwd`
cd "${PROGRAM_DIR}"

#Building Areca's classpath
CLASSPATH="${CLASSPATH}:$PROGRAM_DIR/help"
CLASSPATH="${CLASSPATH}:$PROGRAM_DIR/license"
CLASSPATH="${CLASSPATH}:$PROGRAM_DIR/config"
CLASSPATH="${CLASSPATH}:$PROGRAM_DIR/translations"

CLASSPATH="${CLASSPATH}:${PROGRAM_DIR}/lib/areca.jar"
CLASSPATH="${CLASSPATH}:${PROGRAM_DIR}/lib/mail.jar"
CLASSPATH="${CLASSPATH}:${PROGRAM_DIR}/lib/activation.jar"
CLASSPATH="${CLASSPATH}:${PROGRAM_DIR}/lib/commons-net-1.4.1.jar"
CLASSPATH="${CLASSPATH}:${PROGRAM_DIR}/lib/jakarta-oro-2.0.8.jar"
CLASSPATH="${CLASSPATH}:${PROGRAM_DIR}/lib/org.eclipse.core.commands_3.2.0.I20060605-1400.jar"
CLASSPATH="${CLASSPATH}:${PROGRAM_DIR}/lib/org.eclipse.equinox.common_3.2.0.v20060603.jar"
CLASSPATH="${CLASSPATH}:${PROGRAM_DIR}/lib/org.eclipse.jface_3.2.0.I20060605-1400.jar"
#CLASSPATH="${CLASSPATH}:${PROGRAM_DIR}/lib/org.eclipse.swt.gtk.linux.jar"
CLASSPATH="${CLASSPATH}:${PROGRAM_DIR}/lib/org.eclipse.swt_3.4.0.v3448.jar"
CLASSPATH="${CLASSPATH}:${PROGRAM_DIR}/lib/org.eclipse.swt.gtk.solaris.sparc_3.4.0.v3448.jar"

LIBRARY_PATH="${PROGRAM_DIR}/lib/"
# for Solaris
CLASSPATH="${CLASSPATH}:/usr/share/lib/java/swt.jar"
LIBRARY_PATH="${LIBRARY_PATH}:/usr/lib/swt/"


#Method to locate matching JREs
look_for_java() {
  IFS=$'\n'
  potential_java_dirs=(`ls -1 "$JAVADIR" | sort | tac`)
  IFS=
  for D in "${potential_java_dirs[@]}"; do
    if [[ -d "$JAVADIR/$D" && -x "$JAVADIR/$D/bin/java" ]]; then
      JAVA_PROGRAM_DIR="$JAVADIR/$D/bin/"
      if check_version ; then
        return 0
      else
        return 1
      fi
    fi
  done
  echo "No valid JRE found in ${JAVADIR}."
  return 1
}

#Method to check the current JRE (>=1.4)
check_version() {
  JAVA_HEADER=`${JAVA_PROGRAM_DIR}java -version 2>&1 | head -n 1`
  JAVA_IMPL=`echo ${JAVA_HEADER} | cut -f1 -d' '`
  if [ "$JAVA_IMPL" = "java" ] ; then
    VERSION=`echo ${JAVA_HEADER} | sed "s/java version \"\(.*\)\"/\1/"`
    if echo $VERSION | grep "^1.[0-3]" ; then
      return 1
    else
      return 0
    fi
  else
    return 1
  fi
}

#Locate and test the java executable
if [ "$JAVA_PROGRAM_DIR" == "" ]; then
  if ! command -v java &>/dev/null; then
    if ! look_for_java ; then
      exit 1
    fi
  else
    if ! check_version ; then
      if ! look_for_java ; then
        exit 1
      fi
    fi
  fi
fi

#Launching Areca
echo ${LINE}
echo "The following JRE will be used :"
"${JAVA_PROGRAM_DIR}java" -version
echo ${LINE}
"${JAVA_PROGRAM_DIR}java" -cp "${CLASSPATH}" -Djava.library.path="${LIBRARY_PATH}" com.application.areca.version.VersionCheckLauncher "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" "$9"
echo ${LINE}
echo "Areca terminated."
echo ${LINE}
